// File p3.cpp
// contains the main function

#include <GL/glut.h>
#include <stdlib.h>
#include <string>

// custom header files
#include "title.h"
#include "player1.h"
#include "bullet.h"
#include "boss1.h"
#include "explosion.h"
#include "particle.h"

// standard IO, stream libraries
#include <iostream>
#include <sstream>
using namespace std;

// mouse variables
int mouse_x;
int mouse_y;

// global variables
bool paused = false;
bool game_over = false;
bool auto_camera = true;
float camera_angle = 0.0;

// time variables
clock_t l_start = clock();
clock_t l_end;
clock_t c_start = clock();
clock_t c_end;
clock_t r_start = clock();
clock_t r_end;
clock_t game_start;
clock_t game_end;
double elapsed;

// texture stuff
unsigned int* texList;
unsigned char* image;
float roll = 0.0;

// particle stuff
Particle particles[MAX_PARTICLES];
int particle_num = 0;
#define max_explosions 16   // allow 16 explosions at once
Explosion explosions[max_explosions];
int explosion_num = 0;
bool init_explosions = true;

// title screen variables
Title startup = Title();
bool new_title = true;

// play mode. 1 = title, 2 = gameplay
int mode = 1;

// player1 variables
Player1 p1;
bool new_game = true;
bool new_stage = true;
int p1_axis, p1_direction;
bool p1_stop = true;
#define p1_max_shots 48
Bullet p1shots[p1_max_shots];
bool init_p1shots = true;
int p1_shot_num = 0;

// enemy variables
#define enemy_max_shots 96
Bullet enemyShots[enemy_max_shots];
bool init_enemy_shots = true;
int enemy_shot_num = 0;

// boss variables
Boss1 level1Boss;
bool bossDestroyed = false;

string intToString( int num )
{
  ostringstream sout;
  sout << num;
  return sout.str();
}

void outputString( float x, float y, string text )
{
  int length;

  length = text.size();
  glRasterPos2f( x, y );
  for ( int i = 0; i < length; i++ )
  {
    glutBitmapCharacter( GLUT_BITMAP_9_BY_15, text[i] );
  }
}

void p1HUD()
{
  if ( !game_over ) game_end = clock();
  elapsed = ( game_end - game_start ) / ( CLOCKS_PER_SEC / (double) 1000.0 );
  int seconds = (int) elapsed / 1000;
  int milliseconds = (int) ( ( (int) elapsed % 1000 ) / 10 );
  glPushMatrix();
    glOrtho( -32, 32, -24, 24, 0, 20 );
    glColor3f( 0.8, 0.8, 0.8 );
    outputString( -30, 22, "Score" );
    outputString( -30, 20, intToString( p1.getScore() ) );
    outputString( -30, -20, "Left" );
    if ( p1.getLives()-1 >= 0 )
    outputString( -30, -22, intToString( p1.getLives()-1 ) );
    else outputString( -30, -22, "0" );
    outputString( -20, 22, "Enemy Life" );
    outputString( 22, 22, "Time" );
    if ( auto_camera )
    {
      outputString( -3.0, 2.0, "WARNING!" );
      outputString( -9.0, 0.0, "Large craft approaching!" );
      outputString( -7.0, -1.5, "Attack and Destroy!" );
    }
    if ( !auto_camera )
    {
      if ( seconds < 10 ) outputString( 24, 20, intToString( seconds ) );
      else if ( seconds < 100 ) outputString( 23, 20, intToString( seconds ) );
      else
      outputString( 22, 20, intToString( seconds ) );
      outputString( 25, 20, "\"" );
      if ( milliseconds < 10 )
      {
        outputString( 26, 20, "0" );
        outputString( 27, 20, intToString( milliseconds ) );
      }
      else
      outputString( 26, 20, intToString( milliseconds ) );
    }

    // print "PAUSED" in middle of screen if game is paused
    if ( paused ) outputString( -2.5, 0.0, "PAUSED" );

    // print "Game Over" in middle of screen if player has no lives left
    if ( game_over && !bossDestroyed )
    {
      outputString( -3.0, 0.0, "Game Over" );
    }
    else if ( bossDestroyed )
    {
      int time_bonus = ( 180 - seconds ) * 100;
      if ( time_bonus < 0 ) time_bonus = 0;
      int life_bonus = ( p1.getLives() -1 ) * 5000;
      outputString( -7.5, 5.0, "Enemy destroyed!" );
      outputString( -9.0, 2.5, "Time Bonus:" );
      outputString( 3.0, 2.5, intToString( time_bonus ) );
      outputString( -9.0, 1.0, "Life Bonus:" );
      outputString( 3.0, 1.0, intToString( life_bonus ) );
      outputString( -9.0, -2.0, "Final Score:" );
      outputString( 3.0, -2.0, intToString( p1.getScore() + life_bonus + time_bonus ) );
    }

    glColor3f( 0.2, 0.6, 0.2 );
    glLineWidth(1);
    glBegin( GL_LINE_LOOP );
      glVertex2f( (float) -20, (float) 21 );
      glVertex2f( (float) 20, (float) 21 );
      glVertex2f( (float) 20, (float) 19.5 );
      glVertex2f( (float) -20, (float) 19.5 );
    glEnd();
    glLineWidth(10);
    glColor3f( 0.8, 0.8, 0.2 );
    glBegin( GL_LINES );
      glVertex2f( (float) -19.5, 20.2 );
      glVertex2f( -19.5 + (level1Boss.get_health() * (float) 39.0 / level1Boss.getMaxLife() ), 20.2 );
    glEnd();
    glColor3f( 0.9, 0.2, 0.2 );
    glBegin( GL_LINES );
      glVertex2f( (float) 19.5, 20.2 );
      glVertex2f( -19.5 + (level1Boss.get_health() * (float) 39.0 / level1Boss.getMaxLife() ), 20.2 );
    glEnd();
  glPopMatrix();
}

void idleTitle()
{
  startup.set_angle();
  glutPostRedisplay();
}

void idleScene()
{
  int s;
  int l;

  // roll the background texture
  roll = roll + 0.001;
  if ( roll == 100 ) roll = 0.0;

  // moving the player 1 ship
  if ( p1_stop ) p1.setRoll( 0, 1 );
  else
  {
    if ( p1_axis == 0 )
    {
      if ( p1_direction == 0 && p1.getX() > -19 )
      {
        p1.setPos( -0.035, 0.0 );
        p1.setRoll( 0, 0 );
      }
      else if ( p1_direction == 1 && p1.getX() < 19 )
      {
        p1.setPos( 0.035, 0.0 );
        p1.setRoll( 1, 0 );
      }
    }
    else
    {
      if ( p1_direction == 0 && p1.getY() > -21 )
      {
        p1.setPos( 0.0, -0.045 );
        p1.setRoll( 0, 1 );
      }
      else if ( p1_direction == 1 && p1.getY() < 21 )
      {
        p1.setPos( 0.0, 0.045 );
	p1.setRoll( 0, 1 );
      }
    }
  }

  // moving player 1 bullets
  for ( int b = 0; b < p1_max_shots; b++ )
  {
    p1shots[b].set_pos();
    // check if bullet should be erased
    if ( p1shots[b].get_x_pos() < -25 ||
         p1shots[b].get_x_pos() > 25 ||
         p1shots[b].get_y_pos() < -25 ||
         p1shots[b].get_y_pos() > 25 )
    {
      p1shots[b] = Bullet();
    }
    // check if bullet hit level1Boss
    if ( bossDestroyed == false )
    {
      if ( p1shots[b].get_x_pos() >= level1Boss.getX() - level1Boss.get_hitX() &&
           p1shots[b].get_x_pos() <= level1Boss.getX() + level1Boss.get_hitX() &&
           p1shots[b].get_y_pos() >= level1Boss.getY() - level1Boss.get_hitY() &&
           p1shots[b].get_y_pos() <= level1Boss.getY() + level1Boss.get_hitY() )
      {
        level1Boss.set_health( p1shots[b].get_power() );
        p1.setScore( 10 );
        p1shots[b] = Bullet();
      }
    }
  }

  // moving the level 1 boss
  if ( bossDestroyed == false )
  {
    if ( level1Boss.get_movement() == 0 )
    {
      if ( level1Boss.getX() <= -7.0 ) level1Boss.set_movement(1);
      else level1Boss.set_pos( -0.025, 0.0 );
    }
    else
    {
      if ( level1Boss.getX() >= 7.0 ) level1Boss.set_movement(0);
      else level1Boss.set_pos( 0.025, 0.0 );
    }
    if ( level1Boss.get_health() <= 0 && !bossDestroyed )
    {
      // draw a big explosion DAIBAKUHATSU!!!!
      init_explosions = false;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 1.0,
                                             level1Boss.getY() + 2.5 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 1.0,
                                             level1Boss.getY() - 2.5 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 5.0,
                                             level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 5.0,
                                             level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 8.0,
					     level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 8.0,
					     level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 10.0,
					     level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 10.0,
					     level1Boss.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 8.0,
					     level1Boss.getY() + 2.0 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() - 8.0,
					     level1Boss.getY() -2.0 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 8.0,
					     level1Boss.getY() +2.0 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      explosions[explosion_num] = Explosion( level1Boss.getX() + 8.0,
					     level1Boss.getY() -2.0 );
      explosion_num = ( explosion_num + 1 ) % max_explosions;

      level1Boss = Boss1( 0, 0, 0, 0);
      bossDestroyed = true;
      game_over = true;
    }
  }

  // move the enemy bullets
  for ( int c = 0; c < enemy_max_shots; c++ )
  {
    enemyShots[c].set_pos();
    // check if bullet should be erased
    if ( enemyShots[c].get_x_pos() < -25 ||
         enemyShots[c].get_x_pos() > 25 ||
         enemyShots[c].get_y_pos() < -25 ||
         enemyShots[c].get_y_pos() > 25 )
    {
      enemyShots[c] = Bullet();
    }
    // check if bullet hit player 1 ship
    if ( enemyShots[c].get_type() != 0 && !game_over )
    {
      if ( enemyShots[c].get_x_pos() >= p1.getX() - p1.get_hitX() &&
           enemyShots[c].get_x_pos() <= p1.getX() + p1.get_hitX() &&
           enemyShots[c].get_y_pos() >= p1.getY() - p1.get_hitY() &&
           enemyShots[c].get_y_pos() <= p1.getY() + p1.get_hitY() )
      {
        init_explosions = false;
        explosions[explosion_num] = Explosion( p1.getX(), p1.getY() );
        explosion_num = ( explosion_num + 1 ) % max_explosions;
        //explosions[explosion_num].update();
        s = p1.getScore();
        l = p1.getLives();
        if ( l-1 == 0 )
        {
          game_over = true;
          p1.remove_hitbox();
          p1 = Player1( s, l-1 );
	}
	else 
        p1 = Player1( s, l-1 );
        // initialise all enemy shots so player will not be killed immediately
	// by an old bullet which is at the player origin
        for ( int d = 0; d < enemy_max_shots; d++ )
	{
          enemyShots[d] = Bullet();
	}
        p1_stop = true;
      }
    }
  }

  // check if player hitbox has intersected the enemy hitbox
  if ( bossDestroyed == false && !game_over )
  {
    if ( p1.getX() >= level1Boss.getX() - level1Boss.get_hitX() - 1 &&
         p1.getX() <= level1Boss.getX() + level1Boss.get_hitX() + 1 &&
         p1.getY() >= level1Boss.getY() - level1Boss.get_hitY() - 1 &&
         p1.getY() <= level1Boss.getY() + level1Boss.get_hitY() + 1 )
    {
      init_explosions = false;
      explosions[explosion_num] = Explosion( p1.getX(), p1.getY() );
      explosion_num = ( explosion_num + 1 ) % max_explosions;
      //explosions[explosion_num].update();
      s = p1.getScore();
      l = p1.getLives();
      if ( l-1 == 0 )
      {
        game_over = true;
        p1.remove_hitbox();
        p1 = Player1( s, l-1 );
      }
      else 
      p1 = Player1( s, l-1 );
      p1_stop = true;
    }
  }

  // move the explosion particles
  for ( int k = 0; k < max_explosions; k++ )
  {
    explosions[k].update();
  }

  glutPostRedisplay();
}

void MouseCallback( int button, int state, int x, int y )
{
  if ( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
  {
    mouse_x = x;
    mouse_y = y; 
    if ( mode == 1 )
    {
      if ( mouse_x > 260 && mouse_x < 380 &&
           mouse_y > 330 && mouse_y < 368 )
      {
        mode = 2;
        new_game = true;
        paused = false;
        game_over = false;
        p1_stop = true;
        bossDestroyed = false;
        init_p1shots = true;
        init_enemy_shots = true;
        init_explosions = true;
        auto_camera = true;
        camera_angle = 0.0;
        p1 = Player1();
        level1Boss = Boss1();
      }
      else if ( mouse_x > 260 && mouse_x < 380 &&
              mouse_y > 368 && mouse_y < 405 )
      {
        exit(0);
      }
    }
  }
}

void PassiveMotionCallback( int x, int y )
{
  mouse_x = x;
  mouse_y = y;
  if ( mode == 1 )
  {
    if ( mouse_x > 260 && mouse_x < 380 &&
         mouse_y > 330 && mouse_y < 368 )
    {
      startup.set_cursor(0);
      glutIdleFunc( idleTitle );
    }
    else if ( mouse_x > 260 && mouse_x < 380 &&
              mouse_y > 368 && mouse_y < 405 )
    {
      startup.set_cursor(1);
      glutIdleFunc( idleTitle );
    }
  }
  if ( !paused )
  glutPostRedisplay();
}

void KeyboardCallback(unsigned char c, int x, int y)
{
  switch (c)
  {
    case 'q':
      if ( mode == 1 ) exit(0);
      else
      {
        // go to title screen
        mode = 1;
        new_title = true;
        startup.set_cursor(0);
      }
      break;

    case 'w':
      if ( mode == 2 )
      {
        if ( !paused ) glutIdleFunc( NULL );
        else glutIdleFunc( idleScene );
        paused = !paused;
      }
      break;

    case ' ':
      if ( paused == false )
      p1_stop = true;
      break;

    case 'f':
      if ( mode == 1 )
      {
	if ( startup.get_cursor() == 0 )
        {
          mode = 2;
          new_game = true;
          paused = false;
          game_over = false;
          p1_stop = true;
          bossDestroyed = false;
          init_p1shots = true;
          init_enemy_shots = true;
          init_explosions = true;
          auto_camera = true;
          camera_angle = 0.0;
          p1 = Player1();
          level1Boss = Boss1();
	}
	else exit (0);
      }
      else if ( mode == 2 && !game_over && !auto_camera )
      {
        if ( paused == false )
	{
          // add a laser beam to player 1's rendered bullets
          p1shots[p1_shot_num] = Bullet( 1, p1.getX(), p1.getY() + 1.5, 0.0, 0.5, 2 );
          p1_shot_num = (p1_shot_num + 1) % p1_max_shots;
          glutIdleFunc( idleScene );
	}
      }
      break;

    case 'd':
      // adds two wide vulcan bullets to player 1's bullet list
      if ( mode == 2 && !game_over && !auto_camera )
      {
        if ( paused == false )
	{
          p1shots[p1_shot_num] = Bullet( 2, p1.getX() - 0.2, p1.getY() + 1.5,
                                         -0.2, 0.3, 1 );
          p1_shot_num = (p1_shot_num + 1) % p1_max_shots;
          p1shots[p1_shot_num] = Bullet( 2, p1.getX() + 0.2, p1.getY() + 1.5,
                                         0.2, 0.3, 1 );
          p1_shot_num = (p1_shot_num + 1) % p1_max_shots;
          glutIdleFunc( idleScene );
	}
      }
      break;

    case 'r':
      if ( mode == 2 && game_over )
      {
        // go to title screen
        mode = 1;
        new_title = true;
        startup.set_cursor(0);
      }
      break;
  }
  glutPostRedisplay();
}

void SpecialKeyCallback(int key, int x, int y)
{
  if ( key == GLUT_KEY_LEFT )
  {
    if ( mode == 2 && paused == false )
    {
      p1_axis = 0;
      p1_direction = 0;
      p1_stop = false;
      glutIdleFunc( idleScene );
    }
  }

  if ( key == GLUT_KEY_RIGHT )
  {
    if ( mode == 2 && paused == false )
    {
      p1_axis = 0;
      p1_direction = 1;
      p1_stop = false;
      glutIdleFunc( idleScene );
    }
  }

  if ( key == GLUT_KEY_DOWN )
  {
    if ( mode == 1 )
    {
      startup.set_cursor(1);
      glutIdleFunc( idleTitle );
    }
    else if ( mode == 2 && paused == false )
    {
      p1_axis = 1;
      p1_direction = 0;
      p1_stop = false;
      glutIdleFunc( idleScene );
    }
  }

  if ( key == GLUT_KEY_UP )
  {
    if ( mode == 1 )
    {
      startup.set_cursor(0);
      glutIdleFunc( idleTitle );
    }
    if ( mode == 2 && paused == false )
    {
      p1_axis = 1;
      p1_direction = 1;
      p1_stop = false;
      glutIdleFunc( idleScene );
    }
  }

  glutPostRedisplay();
}

unsigned char* loadPNM( const char *filename,
     unsigned int& width, unsigned int &height,
     unsigned int &numComponents )
{
  FILE          *imgFile;
  char          buf[1024];
  char          type;
  int           i, j, k, l;
  unsigned char    *imageData;

  imgFile= fopen( filename, "rb" );
  if( errno!= 0 )
    return NULL;
  // read PNM magic number (P1 to P6)
  if( fscanf( imgFile, "P%c \n", &type )!= 1 || type< '1' || type> '8')
  {
    fclose( imgFile );
    return NULL;
  }
  // skip comments
  while( fscanf( imgFile, "#%[^\n]\n", buf ) )
    ;
  // read width
  fscanf( imgFile, "%d", &width );
  /* skip comments */
  while( fscanf( imgFile, "#%[^\n]\n", buf ) )
    ;
  /* read height */
  fscanf( imgFile, "%d", &height );
  /* skip comments */
  while( fscanf( imgFile, "#%[^\n]\n", buf ) )
    ;
  /* skip max. component and exactly one whitespace */
  fscanf( imgFile, "%*d%*c" );

  switch( type )
  {
  case '1': // ASCII bitmap
  case '4': // binary bitmap
    cerr << "Bitmaps not implemented\n";
    fclose( imgFile );
    return NULL;
  case '2': // ASCII greymap
    imageData= new unsigned char[width*height];
    for( i= 0 ; i< height ; i++ )
      for( j= 0 ; j< width ; j++ )
      {
    fscanf( imgFile, "%d", &l );
    imageData[i*width+j]= l;
      }
    numComponents= 1;
    break;
  case '3': // ASCII RGB
    imageData= new unsigned char[width*height*3];
    for( i= 0 ; i< height ; i++ )
      for( j= 0 ; j< width ; j++ )
    for( k= 0 ; k< 3 ; k++ )
    {
      fscanf( imgFile, "%d", &l );
      imageData[(i*width+j)*3+k]= l;
    }
    numComponents= 3;
    break;
  case '5': // binary greymap
    imageData= new unsigned char[width*height];
    fread( imageData, 1, width*height, imgFile );
    numComponents= 1;
    break;
  case '6': // binary RGB
    imageData= new unsigned char[width*height*3];
    fread( imageData, 1, width*height*3, imgFile );
    numComponents= 3;
    break;
  }
  fclose( imgFile );

  return imageData;
} 

void DisplayCallback()
{
  // clear the color buffer
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  // set up camera
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  switch ( mode )
  {

    case 1:
      glOrtho( -32, 32, -24, 24, 0, 20 );
      break;

    case 2:
      p1HUD();
      gluPerspective( 45, 1.0, 0.1, 100.0 );
      break;
  }

  // Begin modelling the scene
  glMatrixMode( GL_MODELVIEW );

  // initialise bullets if needed
  if ( init_p1shots )
  {
    for ( int i = 0; i < p1_max_shots; i++ )
    {
      // make a default bullet
      p1shots[i] = Bullet();
    }
    init_p1shots = false;
  }
  if ( init_enemy_shots )
  {
    for ( int j = 0; j < enemy_max_shots; j++ )
    {
      enemyShots[j] = Bullet();
    }
    init_enemy_shots = false;
  }
  if ( init_explosions )
  {
    for ( int e = 0; e < max_explosions; e++ )
    {
      explosions[e] = Explosion();
    }
    init_explosions = false;
  }

  glLoadIdentity();

  switch ( mode )
  {
    case 1:

      glPushMatrix();
      if ( new_title )
      {
        new_title = false;

        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        unsigned int w, h, components;
        image =
          loadPNM("tekkai.bmp.ppm",w,h,components);
        glBindTexture(GL_TEXTURE_2D, texList[0]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, w, h, GL_RGB, GL_UNSIGNED_BYTE, image);

        image =
          loadPNM("treetex.jpg.ppm",w,h,components);
        glBindTexture(GL_TEXTURE_2D, texList[1]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, w, h, GL_RGB, GL_UNSIGNED_BYTE, image);

	/*
        image =
          loadPNM("skytex.jpg.ppm",w,h,components);
        glBindTexture(GL_TEXTURE_2D, texList[2]);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, w, h, GL_RGB, GL_UNSIGNED_BYTE, image);
	*/

        startup.set_tex( texList );

        glutIdleFunc( idleTitle );
      }
      startup.drawTitle();
      glPopMatrix();

      break;

    case 2:
      if ( new_game ) 
      {
        p1 = Player1();
        new_game = false;
        glutIdleFunc( idleScene );
      }

      glPushMatrix();
        glTranslatef( 0.0, 0.0, -55.0 );
        glTranslatef( 0.0, 1.8, 0.0 );
        glRotatef( -15.0, 1.0, 0.0, 0.0 );

        if ( auto_camera )
        {
          glRotatef( camera_angle, 0.0, 0.0, 1.0 );
          camera_angle += 0.25;
          if ( camera_angle == 360 )
          {
            auto_camera = false;
            game_start = clock();
	  }
	}

        // rolling texture background
        glColor3f( 0.8, 0.8, 0.8 );
        glPushMatrix();
          glTranslatef( 0.0, 0.0, -15.0 );
          GLfloat vertices[4][2] =
          { { -50.0, 50.0 }, { 50.0, 50.0 },
            { 50.0, -50.0 }, { -50.0, -50.0 }
          };
          glEnable( GL_TEXTURE_2D );
          glBindTexture( GL_TEXTURE_2D, texList[1] );
          glBegin( GL_QUADS );
            glTexCoord2f( 0.0, -(roll + 1) );
            glVertex2fv( vertices[0] );
            glTexCoord2f( 1.0, -(roll + 1) );
            glVertex2fv( vertices[1] );
            glTexCoord2f( 1.0, -roll );
            glVertex2fv( vertices[2] );
            glTexCoord2f( 0.0, -roll );
            glVertex2fv( vertices[3] );
	  glEnd();
          glDisable( GL_TEXTURE_2D );
        glPopMatrix();

	/*
        // textured sphere as sky
        glPushMatrix();
	glTranslatef( 0.0, 0.0, 40.0 );
	  glEnable( GL_TEXTURE_2D );
          glBindTexture( GL_TEXTURE_2D, texList[2] );
          GLUquadricObj* sphere = gluNewQuadric();
          gluQuadricTexture( sphere, GL_TRUE );
          glRotatef( 90.0, 1.0, 0.0, 0.0 );
          gluSphere( sphere, 75, 36, 36 );
	  gluDeleteQuadric(sphere);
	  glDisable( GL_TEXTURE_2D );
	glPopMatrix();
	*/

        // draw player 1's ship
        glPushMatrix();
          glTranslatef( p1.getX(), 0.0, 0.0 );
          glTranslatef( 0.0, p1.getY(), 0.0 );
          glRotatef( -p1.getRoll(), 0.0, 1.0, 0.0 );
          if ( p1.getLives() > 0 ) p1.drawPlayer1();
        glPopMatrix();

        // draw player 1's bullets
        for ( int b = 0; b < p1_max_shots; b++ )
        {
          if ( p1shots[b].get_type() != 0 )
          {
            glPushMatrix();
              glTranslatef( p1shots[b].get_x_pos(),
                            p1shots[b].get_y_pos(), 0.0 );
              (p1shots[b]).drawBullet();
            glPopMatrix();
          }
        }

        // draw boss enemy
        glPushMatrix();
	  glTranslatef( level1Boss.getX(), level1Boss.getY(), 0.0 );
          level1Boss.drawBoss1();
        glPopMatrix();

        // see if the boss can fire any bullets
        if ( bossDestroyed == false && !auto_camera )
	{
          // centre main cannon
          c_end = clock();
          elapsed = ( c_end - c_start ) / ( CLOCKS_PER_SEC / (double) 1000.0 );
          if ( elapsed > 575 )
	  {
            // centre main cannon
            enemyShots[enemy_shot_num] =
              Bullet( 4, level1Boss.getX(), level1Boss.getY() - 8.0, 0.0, -0.1, 1 );
            enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // centre left cannon
            enemyShots[enemy_shot_num] =
              Bullet( 4, level1Boss.getX() - 2.3, level1Boss.getY() - 7.8, -0.02, -0.08, 1 );
            enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // centre right cannon
            enemyShots[enemy_shot_num] =
              Bullet( 4, level1Boss.getX() + 2.3, level1Boss.getY() - 7.8, 0.02, -0.08, 1 );
            enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // centre far left cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 4, level1Boss.getX() - 4.2, level1Boss.getY() - 7.6, -0.045, -0.04, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // centre far right cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 4, level1Boss.getX() + 4.2, level1Boss.getY() - 7.6, 0.045, -0.04, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;

            // aimed bullet
            if ( p1.getLives() > 0 )
	    {
              enemyShots[enemy_shot_num] =
	        Bullet( 4, level1Boss.getX(), level1Boss.getY() - 4.0, 
		        ( p1.getX() - level1Boss.getX() ) / 475,
                        ( p1.getY() - level1Boss.getY() ) / 475, 1 );
              enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
	    }

            c_start = clock();
	  }

          l_end = clock();
          elapsed = ( l_end - l_start ) / ( CLOCKS_PER_SEC / (double) 1000.0 );
	  if ( elapsed > 550 )
	  {
            // left main cannon
            enemyShots[enemy_shot_num] =
              Bullet( 3, level1Boss.getX() - 10.0, level1Boss.getY() - 7.8, 0.0, -0.075, 1 );
            enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // left left cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 3, level1Boss.getX() - 14.0, level1Boss.getY() - 7.0, -0.018, -0.065, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // left right cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 3, level1Boss.getX() - 6.0, level1Boss.getY() - 7.0, 0.018, -0.065, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;

            l_start = clock();
	  }

          r_end = clock();
          elapsed = ( r_end - r_start ) / ( CLOCKS_PER_SEC / (double) 1000.0 );
	  if ( elapsed > 575 )
	  {
            // right main cannon
            enemyShots[enemy_shot_num] =
              Bullet( 3, level1Boss.getX() + 10.0, level1Boss.getY() - 7.8, 0.0, -0.075, 1 );
            enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // right left cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 3, level1Boss.getX() + 6.0, level1Boss.getY() - 7.0, -0.018, -0.065, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;
            // right right cannon
            enemyShots[enemy_shot_num] =
	      Bullet( 3, level1Boss.getX() + 14.0, level1Boss.getY() - 7.0, 0.018, -0.065, 1 );
	    enemy_shot_num = ( enemy_shot_num + 1 ) % enemy_max_shots;

            r_start = clock();
	  }
	}

        // draw enemy bullets
        for ( int c = 0; c < enemy_max_shots; c++ )
        {
          if ( enemyShots[c].get_type() != 0 && !bossDestroyed )
          {
            glPushMatrix();
              glTranslatef( enemyShots[c].get_x_pos(),
                            enemyShots[c].get_y_pos(), 0.0 );
              (enemyShots[c]).drawBullet();
            glPopMatrix();
          }
        }

	
	// draw any explosions
        if ( !init_explosions )
	{
	  for ( int d = 0; d < max_explosions; d++ )
	  {
            if ( explosions[d].getX() != 0.0 )
	    {
              glPushMatrix();
                explosions[d].drawExplosion();
              glPopMatrix();
	    }
	  }
	}

      glPopMatrix();

      break;

  }

  // draw the buffer to the screen
  glutSwapBuffers();
}

int main(int argc, char **argv)
{
  // create window and rendering context
  glutInit( &argc, argv );
  glutInitDisplayMode( GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE );
  glutInitWindowSize( 640, 500 );
  glutCreateWindow( "Tekkai" );
  
  // register display callback
  glutDisplayFunc( DisplayCallback );
  glutKeyboardFunc( KeyboardCallback );
  glutSpecialFunc( SpecialKeyCallback );
  glutMouseFunc( MouseCallback );
  glutPassiveMotionFunc( PassiveMotionCallback );
  
  glViewport( 0, 0, 640, 480 );
  glEnable( GL_DEPTH_TEST );

  // textures stuff
  texList = new unsigned int[3];
  glGenTextures( 3, texList );
  
  // lighting stuff
  //GLfloat ambient[] = {0.1, 0.1, 0.1, 1.0};
  GLfloat ambient[] = { 0.3, 0.3, 0.3, 1.0 };
  GLfloat diffuse[] = {0.4, 0.4, 0.4, 1.0};
  GLfloat specular[] = {1.0, 1.0, 1.0, 1.0};
  GLfloat position0[] = {1.0, 1.0, 1.0, 0.0};
  glLightfv( GL_LIGHT0, GL_POSITION, position0 );
  glLightfv( GL_LIGHT0, GL_AMBIENT, ambient );
  glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse );
  glLightfv( GL_LIGHT0, GL_SPECULAR, specular );
  GLfloat position1[] = {-1.0, -0.5, -1.0, 0.0};
  glLightfv( GL_LIGHT1, GL_POSITION, position1 );
  glLightfv( GL_LIGHT1, GL_AMBIENT, ambient );
  glLightfv( GL_LIGHT1, GL_DIFFUSE, diffuse );
  glLightfv( GL_LIGHT1, GL_SPECULAR, specular );

  glEnable( GL_LIGHTING );
  glEnable( GL_LIGHT0 );
  glEnable( GL_LIGHT1 );
  glEnable( GL_COLOR_MATERIAL );

  // pass control over to GLUT
  glutMainLoop();
  
  return 0;       // never reached
}

