// File title.cpp
// implements the title screen

#include <GL/glut.h>
#include <stdlib.h>
#include <string>

#include "title.h"

using namespace std;

// default constructor
Title::Title( void )
{
  cursor = 0;
  c_size = 1.0;
  angle = 0;
  c_inc = true;
  start = "START";
  quit = "QUIT";
  textures = new unsigned int[3];
}

int Title::get_cursor()
{
  return cursor;
}

void Title::set_cursor( int direction )
{
  if ( direction == 1 && cursor == 0 ) cursor = 1;
  else if ( direction == 0 && cursor == 1 ) cursor = 0;
}

void Title::set_angle()
{
  angle += 0.25;
  if ( angle == 90 ) angle = 0;

  if ( c_inc )
  {
    c_size += 0.0125;
  }
  else
  {
    c_size -= 0.0125;
  }

  if ( c_size <= 1.0 ) c_inc = true;
  else if ( c_size >= 2.5 ) c_inc = false;
}

void Title::outputString( float x, float y, string text )
{
  int length;

  length = text.size();
  glRasterPos2f( x, y );
  for ( int i = 0; i < length; i++ )
  {
    glutBitmapCharacter( GLUT_BITMAP_TIMES_ROMAN_24, text[i] );
  }
}

void Title::set_tex( unsigned int* texList )
{
  textures = texList;
}

void Title::drawTitle()
{
  GLfloat vertices[4][2] =
  { { -24.0, 20.0 }, { 24.0, 20.0 },
    { 24.0, -4.0 }, { -24.0, -4.0 }
  };

  // title graphics
  glColor3f( 0.9, 0.9, 0.9 );
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, textures[0] );
  glBegin( GL_QUADS );
    glTexCoord2f( 0.0, 0.0 );
    glVertex2fv( vertices[0] );
    glTexCoord2f( 1.0, 0.0 );
    glVertex2fv( vertices[1] );
    glTexCoord2f( 1.0, 1.0 );
    glVertex2fv( vertices[2] );
    glTexCoord2f( 0.0, 1.0 );
    glVertex2fv( vertices[3] );
  glEnd();
  glDisable( GL_TEXTURE_2D );

  // menu options
  glColor3f( 0.9, 0.9, 0.9 );
  outputString( (float) -3.75, (float) -10, start );
  outputString( (float) -3, (float) -13.5, quit );

  glColor3f( 0.4, 0.4, 1.0 );
  if ( cursor == 0 )
  {
    glPushMatrix();
      glTranslatef( -6.0, -9.25, 0.0 );
      glRotatef( angle, 0.0, 0.0, 1.0 );
      glutSolidCube( c_size );
    glPopMatrix();
    glPushMatrix();
      glTranslatef( 6.0, -9.25, 0.0 );
      glRotatef( -angle, 0.0, 0.0, 1.0 );
      glutSolidCube( c_size );
    glPopMatrix();
  }
  else 
  {
    glPushMatrix();
      glTranslatef( -5.0, -12.75, 0.0 );
      glRotatef( angle, 0.0, 0.0, 1.0 );
      glutSolidCube( c_size );
    glPopMatrix();
    glPushMatrix();
      glTranslatef( 5.0, -12.75, 0.0 );
      glRotatef( -angle, 0.0, 0.0, 1.0 );
      glutSolidCube( c_size );
    glPopMatrix();
  }

}
